@@ -365,6 +365,15 @@ int clientSendRemove(struct client *c, unsigned id)
 	return 0;
 }
 
+int clientSendTransfer(struct client *c, unsigned id)
+{
+	tracef("client send transfer fd %d id %u", c->fd, id);
+	struct request_transfer request;
+	request.id = id;
+	REQUEST(transfer, TRANSFER);
+	return 0;
+}
+
 int clientRecvEmpty(struct client *c)
 {
 	tracef("client recv empty fd %d", c->fd);
@@ -372,3 +381,13 @@ int clientRecvEmpty(struct client *c)
 	RESPONSE(empty, EMPTY);
 	return 0;
 }
+
+int clientRecvFailure(struct client *c, uint64_t *code, const char **msg)
+{
+	tracef("client recv failure fd %d", c->fd);
+	struct response_failure response;
+	RESPONSE(failure, FAILURE);
+	*code = response.code;
+	*msg = response.message;
+	return 0;
+}
