@@ -61,6 +61,124 @@ static int add_pending_pages(struct fsm *f,
 	return 0;
 }
 
+static int databaseReadLock(struct db *db)
+{
+    if (!db->read_lock) {
+	    db->read_lock = 1;
+	    return 0;
+    } else {
+	    return -1;
+    }
+}
+
+static int databaseReadUnlock(struct db *db)
+{
+    if (db->read_lock) {
+	    db->read_lock = 0;
+	    return 0;
+    } else {
+	    return -1;
+    }
+}
+
+static void maybeCheckpoint(struct db *db)
+{
+	tracef("maybe checkpoint");
+	struct sqlite3_file *main_f;
+	struct sqlite3_file *wal;
+	volatile void *region;
+	sqlite3_int64 size;
+	unsigned page_size;
+	unsigned pages;
+	int wal_size;
+	int ckpt;
+	int i;
+	int rv;
+
+	/* Don't run when a snapshot is busy. Running a checkpoint while a snapshot
+	 * is busy will result in illegal memory accesses by the routines that try
+	 * to access database page pointers contained in the snapshot. */
+	rv = databaseReadLock(db);
+	if (rv != 0) {
+		tracef("busy snapshot %d", rv);
+		return;
+	}
+
+	assert(db->follower == NULL);
+	rv = db__open_follower(db);
+	if (rv != 0) {
+		tracef("open follower failed %d", rv);
+		goto err_after_db_lock;
+	}
+
+	page_size = db->config->page_size;
+	/* Get the database wal file associated with this connection */
+	rv = sqlite3_file_control(db->follower, "main", SQLITE_FCNTL_JOURNAL_POINTER,
+				  &wal);
+	assert(rv == SQLITE_OK); /* Should never fail */
+
+	rv = wal->pMethods->xFileSize(wal, &size);
+	assert(rv == SQLITE_OK); /* Should never fail */
+
+	/* Calculate the number of frames. */
+	pages = ((unsigned)size - 32) / (24 + page_size);
+
+	/* Check if the size of the WAL is beyond the threshold. */
+	if (pages < db->config->checkpoint_threshold) {
+		tracef("wal size (%u) < threshold (%u)", pages, db->config->checkpoint_threshold);
+		goto err_after_db_open;
+	}
+
+	/* Get the database file associated with this db->follower connection */
+	rv = sqlite3_file_control(db->follower, "main", SQLITE_FCNTL_FILE_POINTER,
+				  &main_f);
+	assert(rv == SQLITE_OK); /* Should never fail */
+
+	/* Get the first SHM region, which contains the WAL header. */
+	rv = main_f->pMethods->xShmMap(main_f, 0, 0, 0, &region);
+	assert(rv == SQLITE_OK); /* Should never fail */
+
+	rv = main_f->pMethods->xShmUnmap(main_f, 0);
+	assert(rv == SQLITE_OK); /* Should never fail */
+
+	/* Try to acquire all locks. */
+	for (i = 0; i < SQLITE_SHM_NLOCK; i++) {
+		int flags = SQLITE_SHM_LOCK | SQLITE_SHM_EXCLUSIVE;
+
+		rv = main_f->pMethods->xShmLock(main_f, i, 1, flags);
+		if (rv == SQLITE_BUSY) {
+			tracef("busy reader or writer - retry next time");
+			goto err_after_db_open;
+		}
+
+		/* Not locked. Let's release the lock we just
+		 * acquired. */
+		flags = SQLITE_SHM_UNLOCK | SQLITE_SHM_EXCLUSIVE;
+		main_f->pMethods->xShmLock(main_f, i, 1, flags);
+	}
+
+	rv = sqlite3_wal_checkpoint_v2(
+	     db->follower, "main", SQLITE_CHECKPOINT_TRUNCATE, &wal_size, &ckpt);
+	/* TODO assert(rv == 0) here? Which failure modes do we expect? */
+	if (rv != 0) {
+		tracef("sqlite3_wal_checkpoint_v2 failed %d", rv);
+		goto err_after_db_open;
+	}
+	tracef("sqlite3_wal_checkpoint_v2 success");
+
+	/* Since no reader transaction is in progress, we must be able to
+	 * checkpoint the entire WAL */
+	assert(wal_size == 0);
+	assert(ckpt == 0);
+
+err_after_db_open:
+	sqlite3_close(db->follower);
+	db->follower = NULL;
+err_after_db_lock:
+	rv = databaseReadUnlock(db);
+	assert(rv == 0);
+}
+
 static int apply_frames(struct fsm *f, const struct command_frames *c)
 {
         tracef("fsm apply frames");
@@ -153,6 +271,7 @@ static int apply_frames(struct fsm *f, const struct command_frames *c)
 	}
 
 	sqlite3_free(page_numbers);
+	maybeCheckpoint(db);
 	return 0;
 }
 
@@ -174,50 +293,14 @@ static int apply_undo(struct fsm *f, const struct command_undo *c)
 	return 0;
 }
 
+/* Checkpoints used to be coordinated cluster-wide, these days a node
+ * checkpoints independently in `apply_frames`, the checkpoint command becomes a
+ * no-op for modern nodes. */
 static int apply_checkpoint(struct fsm *f, const struct command_checkpoint *c)
 {
-        tracef("apply checkpoint");
-	struct db *db;
-	struct sqlite3_file *file;
-	int size;
-	int ckpt;
-	int rv;
-
-	rv = registry__db_get(f->registry, c->filename, &db);
-	assert(rv == 0); /* We have registered this filename before. */
-
-	/* Use a new connection to force re-opening the WAL. */
-	rv = db__open_follower(db);
-	if (rv != 0) {
-                tracef("open failed %d", rv);
-		return rv;
-	}
-
-	rv = sqlite3_file_control(db->follower, "main",
-				  SQLITE_FCNTL_FILE_POINTER, &file);
-	assert(rv == SQLITE_OK); /* Should never fail */
-
-	/* If there's a checkpoint lock in place, we must be the node that
-	 * originated the checkpoint command in the first place, let's release
-	 * it. */
-	file->pMethods->xShmLock(file, 1 /* checkpoint lock */, 1,
-				 SQLITE_SHM_UNLOCK | SQLITE_SHM_EXCLUSIVE);
-
-	rv = sqlite3_wal_checkpoint_v2(
-	    db->follower, "main", SQLITE_CHECKPOINT_TRUNCATE, &size, &ckpt);
-	if (rv != 0) {
-                tracef("sqlite wal checkpoint failed %d", rv);
-		return rv;
-	}
-
-	sqlite3_close(db->follower);
-	db->follower = NULL;
-
-	/* Since no reader transaction is in progress, we must be able to
-	 * checkpoint the entire WAL */
-	assert(size == 0);
-	assert(ckpt == 0);
-
+	(void) f;
+	(void) c;
+	tracef("apply no-op checkpoint");
 	return 0;
 }
 
@@ -288,7 +371,7 @@ static int encodeSnapshotHeader(unsigned n, struct raft_buffer *buf)
 	header.format = SNAPSHOT_FORMAT;
 	header.n = n;
 	buf->len = snapshotHeader__sizeof(&header);
-	buf->base = raft_malloc(buf->len);
+	buf->base = sqlite3_malloc64(buf->len);
 	if (buf->base == NULL) {
 		return RAFT_NOMEM;
 	}
@@ -298,19 +381,20 @@ static int encodeSnapshotHeader(unsigned n, struct raft_buffer *buf)
 }
 
 /* Encode the given database. */
-static int encodeDatabase(struct db *db, struct raft_buffer bufs[2])
+static int encodeDatabase(struct db *db, struct raft_buffer r_bufs[], uint32_t n)
 {
 	struct snapshotDatabase header;
 	sqlite3_vfs *vfs;
 	uint32_t database_size = 0;
 	uint8_t *page;
 	void *cursor;
+	struct dqlite_buffer *bufs = (struct dqlite_buffer*) r_bufs;
 	int rv;
 
 	header.filename = db->filename;
 
 	vfs = sqlite3_vfs_find(db->config->name);
-	rv = VfsSnapshot(vfs, db->filename, &bufs[1].base, &bufs[1].len);
+	rv = VfsShallowSnapshot(vfs, db->filename, &bufs[1], n-1);
 	if (rv != 0) {
 		goto err;
 	}
@@ -323,11 +407,11 @@ static int encodeDatabase(struct db *db, struct raft_buffer bufs[2])
 	database_size += (uint32_t)(page[31]);
 
 	header.main_size = database_size * db->config->page_size;
-	header.wal_size = bufs[1].len - header.main_size;
+	header.wal_size = bufs[n-1].len;
 
 	/* Database header. */
 	bufs[0].len = snapshotDatabase__sizeof(&header);
-	bufs[0].base = raft_malloc(bufs[0].len);
+	bufs[0].base = sqlite3_malloc64(bufs[0].len);
 	if (bufs[0].base == NULL) {
 		rv = RAFT_NOMEM;
 		goto err_after_snapshot;
@@ -338,7 +422,8 @@ static int encodeDatabase(struct db *db, struct raft_buffer bufs[2])
 	return 0;
 
 err_after_snapshot:
-	raft_free(bufs[1].base);
+	/* Free the wal buffer */
+	sqlite3_free(bufs[n-1].base);
 err:
 	assert(rv != 0);
 	return rv;
@@ -395,6 +480,77 @@ static int decodeDatabase(struct fsm *f, struct cursor *cursor)
 	return 0;
 }
 
+static unsigned dbNumPages(struct db *db)
+{
+	sqlite3_vfs *vfs;
+	int rv;
+	uint32_t n;
+
+	vfs = sqlite3_vfs_find(db->config->name);
+	rv = VfsDatabaseNumPages(vfs, db->filename, &n);
+	assert(rv == 0);
+	return n;
+}
+
+/* Determine the total number of raft buffers needed for a snapshot */
+static unsigned snapshotNumBufs(struct fsm *f)
+{
+	struct db *db;
+	queue *head;
+	unsigned n = 1; /* snapshot header */
+
+	QUEUE__FOREACH(head, &f->registry->dbs)
+	{
+		n += 2; /* database header & wal */
+		db = QUEUE__DATA(head, struct db, queue);
+		n += dbNumPages(db); /* 1 buffer per page (zero copy) */
+	}
+
+	return n;
+}
+
+/* An example array of snapshot buffers looks like this:
+ *
+ * bufs:  SH DH1 P1 P2 P3 WAL1 DH2 P1 P2 WAL2
+ * index:  0   1  2  3  4    5   6  7  8    9
+ *
+ * SH:   Snapshot Header
+ * DHx:  Database Header
+ * Px:   Database Page (not to be freed)
+ * WALx: a WAL
+ * */
+static void freeSnapshotBufs(struct fsm *f,
+			     struct raft_buffer bufs[],
+			     unsigned n_bufs)
+{
+	queue *head;
+	struct db *db;
+	unsigned i;
+
+	if (bufs == NULL || n_bufs == 0) {
+		return;
+	}
+
+	/* Free snapshot header */
+	sqlite3_free(bufs[0].base);
+
+	i = 1;
+	/* Free all database headers & WAL buffers */
+	QUEUE__FOREACH(head, &f->registry->dbs)
+	{
+		if (i == n_bufs) {
+		    break;
+		}
+		db = QUEUE__DATA(head, struct db, queue);
+		/* i is the index of the database header */
+		sqlite3_free(bufs[i].base);
+		/* i is now the index of the next database header (if any) */
+		i += 1 /* db header */ + dbNumPages(db) + 1 /* WAL */;
+		/* free WAL buffer */
+		sqlite3_free(bufs[i-1].base);
+	}
+}
+
 static int fsm__snapshot(struct raft_fsm *fsm,
 			 struct raft_buffer *bufs[],
 			 unsigned *n_bufs)
@@ -402,30 +558,37 @@ static int fsm__snapshot(struct raft_fsm *fsm,
 	struct fsm *f = fsm->data;
 	queue *head;
 	struct db *db;
-	unsigned n = 0;
+	unsigned n_db = 0;
 	unsigned i;
 	int rv;
 
 	/* First count how many databases we have and check that no transaction
-	 * is in progress. */
+	 * nor checkpoint nor other snapshot is in progress. */
 	QUEUE__FOREACH(head, &f->registry->dbs)
 	{
 		db = QUEUE__DATA(head, struct db, queue);
-		if (db->tx_id != 0) {
+		if (db->tx_id != 0 || db->read_lock) {
 			return RAFT_BUSY;
 		}
-		n++;
+		n_db++;
 	}
 
-	*n_bufs = 1;      /* Snapshot header */
-	*n_bufs += n * 2; /* Database header an content */
-	*bufs = raft_malloc(*n_bufs * sizeof **bufs);
+	/* Lock all databases, preventing the checkpoint from running */
+	QUEUE__FOREACH(head, &f->registry->dbs)
+	{
+		db = QUEUE__DATA(head, struct db, queue);
+		rv = databaseReadLock(db);
+		assert(rv == 0);
+	}
+
+	*n_bufs = snapshotNumBufs(f);
+	*bufs = sqlite3_malloc64(*n_bufs * sizeof **bufs);
 	if (*bufs == NULL) {
 		rv = RAFT_NOMEM;
 		goto err;
 	}
 
-	rv = encodeSnapshotHeader(n, &(*bufs)[0]);
+	rv = encodeSnapshotHeader(n_db, &(*bufs)[0]);
 	if (rv != 0) {
 		goto err_after_bufs_alloc;
 	}
@@ -435,27 +598,81 @@ static int fsm__snapshot(struct raft_fsm *fsm,
 	QUEUE__FOREACH(head, &f->registry->dbs)
 	{
 		db = QUEUE__DATA(head, struct db, queue);
-		rv = encodeDatabase(db, &(*bufs)[i]);
+		/* database_header + num_pages + wal */
+		unsigned n = 1 + dbNumPages(db) + 1;
+		rv = encodeDatabase(db, &(*bufs)[i], n);
 		if (rv != 0) {
 			goto err_after_encode_header;
 		}
-		i += 2;
+		i += n;
 	}
 
+	assert(i == *n_bufs);
 	return 0;
 
 err_after_encode_header:
-	do {
-		raft_free((*bufs)[i].base);
-		i--;
-	} while (i > 0);
+	freeSnapshotBufs(f, *bufs, i);
 err_after_bufs_alloc:
-	raft_free(*bufs);
+	sqlite3_free(*bufs);
 err:
+	QUEUE__FOREACH(head, &f->registry->dbs)
+	{
+		db = QUEUE__DATA(head, struct db, queue);
+		databaseReadUnlock(db);
+	}
 	assert(rv != 0);
 	return rv;
 }
 
+static int fsm__snapshot_finalize(struct raft_fsm *fsm,
+				  struct raft_buffer *bufs[],
+				  unsigned *n_bufs)
+{
+	struct fsm *f = fsm->data;
+	queue *head;
+	struct db *db;
+	unsigned n_db;
+	struct snapshotHeader header;
+	int rv;
+
+	if (bufs == NULL) {
+		return 0;
+	}
+
+	/* Decode the header to determine the number of databases. */
+	struct cursor cursor = {(*bufs)[0].base, (*bufs)[0].len};
+	rv = snapshotHeader__decode(&cursor, &header);
+	if (rv != 0) {
+		tracef("decode failed %d", rv);
+		return -1;
+	}
+	if (header.format != SNAPSHOT_FORMAT) {
+		tracef("bad format");
+		return -1;
+	}
+
+	/* Free allocated buffers */
+	freeSnapshotBufs(f, *bufs, *n_bufs);
+	sqlite3_free(*bufs);
+	*bufs = NULL;
+	*n_bufs = 0;
+
+	/* Unlock all databases that were locked for the snapshot, this is safe
+	 * because DB's are only ever added at the back of the queue. */
+	n_db = 0;
+	QUEUE__FOREACH(head, &f->registry->dbs)
+	{
+		if (n_db == header.n) {
+			break;
+		}
+		db = QUEUE__DATA(head, struct db, queue);
+		databaseReadUnlock(db);
+		n_db++;
+	}
+
+	return 0;
+}
+
 static int fsm__restore(struct raft_fsm *fsm, struct raft_buffer *buf)
 {
         tracef("fsm restore");
@@ -483,6 +700,7 @@ static int fsm__restore(struct raft_fsm *fsm, struct raft_buffer *buf)
 		}
 	}
 
+	/* Don't use sqlite3_free as this buffer is allocated by raft. */
 	raft_free(buf->base);
 
 	return 0;
@@ -505,10 +723,11 @@ int fsm__init(struct raft_fsm *fsm,
 	f->pending.page_numbers = NULL;
 	f->pending.pages = NULL;
 
-	fsm->version = 1;
+	fsm->version = 2;
 	fsm->data = f;
 	fsm->apply = fsm__apply;
 	fsm->snapshot = fsm__snapshot;
+	fsm->snapshot_finalize = fsm__snapshot_finalize;
 	fsm->restore = fsm__restore;
 
 	return 0;
