@@ -17,8 +17,7 @@ static int endpointConnect(void *data,
 	*fd = socket(AF_UNIX, SOCK_STREAM, 0);
 	munit_assert_int(*fd, !=, -1);
 	rv = connect(*fd, (struct sockaddr *)&addr, sizeof(sa_family_t) + strlen(address + 1) + 1);
-	munit_assert_int(rv, ==, 0);
-	return 0;
+	return rv;
 }
 
 void test_server_setup(struct test_server *s,
@@ -35,23 +34,24 @@ void test_server_setup(struct test_server *s,
 	memset(s->others, 0, sizeof s->others);
 }
 
-void test_server_tear_down(struct test_server *s)
+void test_server_stop(struct test_server *s)
 {
 	int rv;
 
-	clientClose(&s->client);
-	close(s->client.fd);
+	test_server_client_close(s, &s->client);
 	rv = dqlite_node_stop(s->dqlite);
 	munit_assert_int(rv, ==, 0);
-
 	dqlite_node_destroy(s->dqlite);
+}
 
+void test_server_tear_down(struct test_server *s)
+{
+	test_server_stop(s);
 	test_dir_tear_down(s->dir);
 }
 
 void test_server_start(struct test_server *s)
 {
-	int client;
 	int rv;
 
 	rv = dqlite_node_create(s->id, s->address, s->dir, &s->dqlite);
@@ -63,15 +63,13 @@ void test_server_start(struct test_server *s)
 	rv = dqlite_node_set_connect_func(s->dqlite, endpointConnect, s);
 	munit_assert_int(rv, ==, 0);
 
+	rv = dqlite_node_set_network_latency_ms(s->dqlite, 10);
+	munit_assert_int(rv, ==, 0);
+
 	rv = dqlite_node_start(s->dqlite);
 	munit_assert_int(rv, ==, 0);
 
-	/* Connect a client. */
-	rv = endpointConnect(NULL, s->address, &client);
-	munit_assert_int(rv, ==, 0);
-
-	rv = clientInit(&s->client, client);
-	munit_assert_int(rv, ==, 0);
+	test_server_client_connect(s, &s->client);
 }
 
 struct client *test_server_client(struct test_server *s)
@@ -79,6 +77,31 @@ struct client *test_server_client(struct test_server *s)
 	return &s->client;
 }
 
+void test_server_client_reconnect(struct test_server *s, struct client *c)
+{
+	test_server_client_close(s, c);
+	test_server_client_connect(s, c);
+}
+
+void test_server_client_connect(struct test_server *s, struct client *c)
+{
+	int rv;
+	int fd;
+
+	rv = endpointConnect(NULL, s->address, &fd);
+	munit_assert_int(rv, ==, 0);
+
+	rv = clientInit(c, fd);
+	munit_assert_int(rv, ==, 0);
+}
+
+void test_server_client_close(struct test_server *s, struct client *c)
+{
+	(void) s;
+	clientClose(c);
+	close(c->fd);
+}
+
 static void setOther(struct test_server *s, struct test_server *other)
 {
 	unsigned i = other->id - 1;
